//--------------------------------------------------------------------------------------------------
//
// Title       : Controller
// Design      : mips2000
//
//-------------------------------------------------------------------------------------------------
//
// Description :	Combinational Controller that generates the control signals for the
//					CPU's components. To be implemented by you!
//
//-------------------------------------------------------------------------------------------------
`timescale 1ns / 1ps

module Controller(
	input	[31:0]	Inst,
	input			neg,
	input			zero,
	input			reset,
	output			srcB,
	output	[5:0]	ALUctrl,
	output			regWrite,
	output	[1:0]	wrDataSel,
	output	[1:0]	wrRegSel,
	output	[1:0]	PCSel,
	output			MemRead,
	output			MemWrite
);

  reg	[1:0] PCSel,wrDataSel,wrRegSel; 
  reg	[5:0] ALUctrl;
  reg	srcB, regWrite;
  reg	MemRead, MemWrite;

      // place your controller logic here! 
  parameter ALU		= 6'h00; // op =  0
  parameter LW		= 6'h23; // op = 35
  parameter SW		= 6'h2b; // op = 43
  parameter BEQ		= 6'h04; // op =  4 
  parameter ADDI	= 6'h08; // op =  8
  parameter J		= 6'h02; // op =  2
  parameter JAL		= 6'h03; // op =  3
  parameter HALT	= 6'h3f; // op = 63
  
  //Added stuff
  parameter LUI		= 6'h10; // op = 16
  parameter ORI		= 6'h0d; // op = 13

  parameter shftX 	= 5'hxx; // we won't be using shift amount
  
  parameter r0 		= 5'h00; 
  parameter r1 		= 5'h01;
  parameter r2 		= 5'h02;
  parameter r3 		= 5'h03;
  parameter rz 		= 5'h1f; // register 31 stores a zero

  parameter ADD 	= 6'h20; // funct = 32
  parameter SUB 	= 6'h22; // funct = 34
  parameter AND 	= 6'h24; // funct = 36
  parameter OR 		= 6'h25; // funct = 37
  parameter SLT 	= 6'h2a; // funct = 42
  parameter JR		= 6'h08; // funct = 8
  
  // place your declarations here!
  
  wire [5:0] opcode;
  wire [5:0] aluOp;
  
  assign opcode = Inst[31:26];
  assign aluOp = Inst[5:0];
  
  always @(opcode or neg or zero or reset) begin 
	  if(reset) begin
		  MemWrite = 1'b0;
		  MemRead = 1'b0;
		  wrDataSel = 2'b00;
		  PCSel = 2'b00;
		  srcB = 1'b0;
		  regWrite = 1'b0;
		  wrRegSel = 2'b00;
		  ALUctrl = 6'b000001;
	  end
	  else begin
	  	case(opcode)
		  ALU: begin
			  case(aluOp)
				  ADD: begin ALUctrl = 6'bxxxxxx; PCSel = 2'bxx; end
				  SUB: begin ALUctrl = 6'bxxxxxx; PCSel = 2'bxx; end
				  AND: begin ALUctrl = 6'bxxxxxx; PCSel = 2'bxx; end
				  OR : begin ALUctrl = 6'bxxxxxx; PCSel = 2'bxx; end
				  SLT: begin ALUctrl = 6'bxxxxxx; PCSel = 2'bxx; end
				  JR : begin ALUctrl = 6'bxxxxxx; PCSel = 2'bxx; end
				  default: ALUctrl = 6'bxxxxxx;
			  endcase
			  wrDataSel = 2'bxx;
			  MemWrite = 1'bx;
			  MemRead = 1'bx;
			  PCSel = 2'bxx;
			  srcB = 1'bx;
			  regWrite = 1'bx;
			  wrRegSel = 2'bxx; 
		  end
		  LW: begin
			  wrDataSel = 2'bxx;
			  MemWrite = 1'bx;
			  MemRead = 1'bx;
			  PCSel = 2'bxx;
			  srcB = 1'bx;
			  regWrite = 1'bx;
			  wrRegSel = 2'bxx;
			  ALUctrl = 6'bxxxxxx;
		  end 
		  SW: begin
			  wrDataSel = 2'bxx;
			  MemWrite = 1'bx;
			  MemRead = 1'bx;
			  PCSel = 2'bxx;
			  srcB = 1'bx;
			  regWrite = 1'bx;
			  wrRegSel = 2'bxx;
			  ALUctrl = 6'bxxxxxx;
		  end
		  BEQ: begin
			  wrDataSel = 2'bxx;
			  MemWrite = 1'bx;
			  MemRead = 1'bx;
			  PCSel = {1'bx,zero};
			  srcB = 1'bx;	
			  ALUctrl = 6'bxxxxxx;
			  regWrite = 1'bx;
			  wrRegSel = 2'bxx;
		  end
		  ADDI: begin
			  wrDataSel = 2'bxx;
			  MemWrite = 1'bx;
			  MemRead = 1'bx;
			  PCSel = 2'bxx;
			  srcB = 1'bx;
			  regWrite = 1'bx;
			  wrRegSel = 2'bxx;
			  ALUctrl = 6'bxxxxxx;
		  end
		  J: begin
			  wrDataSel = 2'bxx;
			  MemWrite = 1'bx;
			  MemRead = 1'bx;
			  PCSel = 2'bxx;
			  srcB = 1'bx;
			  regWrite = 1'bx;
			  wrRegSel = 2'bxx;
			  ALUctrl = 6'bxxxxxx;
		  end
		  LUI: begin 
			  wrDataSel = 2'bxx;
			  MemWrite = 1'bx;
			  MemRead = 1'bx;
			  PCSel = 2'bxx;
			  srcB = 1'bx;
			  regWrite = 1'bx;
			  wrRegSel = 2'bxx;
			  ALUctrl = 6'bxxxxxx;
		  end
		  ORI: begin
			  wrDataSel = 2'bxx;
			  MemWrite = 1'bx;
			  MemRead = 1'bx;
			  PCSel = 2'bxx;
			  srcB = 1'bx;
			  regWrite = 1'bx;
			  wrRegSel = 2'bxx;
			  ALUctrl = 6'bxxxxxx; 
		  end
		  JAL: begin				 
			  wrDataSel = 2'bxx;
			  MemWrite = 1'bx;
			  MemRead = 1'bx;
			  PCSel = 2'bxx;
			  srcB = 1'bx;
			  regWrite = 1'bx;
			  wrRegSel = 2'bxx;
			  ALUctrl = 6'bxxxxxx; 
			end
		  HALT: begin
			 $display("HALT");
			 $stop;		  
		  end	 
		  default: begin
		  $display("Invalid Instruction");
		  $stop;
		  end
  		endcase
	 end
  end

endmodule
