//--------------------------------------------------------------------------------------------------
//
// Title       : wrDataMux
// Design      : mips2000
//
//-------------------------------------------------------------------------------------------------
//
// Description :	Select the source for register writes
//					Source is selected by wrDataSel input, has to be set by controller
//
// 00 Out = ALU
// 01 Out = Mem
// 1X Out = Next
//
//-------------------------------------------------------------------------------------------------
`timescale 1 ns / 1 ps

module wrDataMux (
	input	[31:0]	ALU,				// Result from ALU
	input	[31:0]	Mem,				// Read data from memory
	input	[31:0]	Next,				// 'Next' address for PC
	input	[1:0]	wrDataSel,
	output	[31:0]	Out
);

reg	[31:0]	Out;

always@(wrDataSel or Next or Mem or ALU) begin
	case(wrDataSel)
		2'b00: Out = ALU;
		2'b01: Out = Mem;
		2'b10: Out = Next;
		2'b11: Out = Next;
	endcase
end

endmodule
